const ZERO = 48;

/// A class with only static members to make the functions namespaced
class Luhn {
  /// Compute the check digit of the sequence of numbers [value] and return
  /// it as an integer
  ///
  /// Please not that for now, we do not check that the input sequence is
  /// actually made of only digits
  static int computeCheckDigit(String value) {
    final runes = value.runes;
    var sum = 0;
    for (var i = runes.length - 1, j = 0; i >= 0; --i, j++) {
      var digit = runes.elementAt(i) - ZERO;
      if (j % 2 == 0) {
        digit *= 2;
        if (digit > 9) {
          digit -= 9;
        }
      }
      sum += digit;
    }
    if (sum % 10 == 0) {
      return 0;
    }
    return 10 - sum % 10;
  }

  /// Compute the check digit of the numeric sequence [value] and
  /// append it to it, returning a new string with the check digit
  /// attached
  static String computeAndAppendCheckDigit(String value) {
    final checkDigit = computeCheckDigit(value);
    // Concatenate the values
    return '$value$checkDigit';
  }

  /// Returns true if the number sequence [value] has a valid Luhn
  /// check digit
  static bool isValidNumber(String value) {
    final checkDigit = computeCheckDigit(value.substring(0, value.length - 1));
    return value.endsWith('$checkDigit');
  }
}
